/* eslint-disable no-console */
import {
  Modal, ModalService, Select, TextInput, Textarea, Button, Text, Form, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, Http, Router, Loader, IEventParam,
} from '@zeedhi/core';
import {
  IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import ZdRatings from '@/components/ratings/ZdRatings';
import FileUpload from './FileUpload';

export class SolicitacoesController {
  private filtersModal!: Modal;

  public hierarquiasController = Loader.getInstance('HierarquiasController');

  public async createModal() {
    this.filtersModal = await ModalService.createFromJson('Modais/filtros-minhasSolicitacoes', true);
  }

  public destroyModal() {
    this.filtersModal?.destroy();
  }

  public openModal() {
    this.filtersModal.show();
  }

  public closeModal() {
    this.filtersModal.hide();
  }

  private dataFilter: IDictionary = {
    selectName: '',
    selectLocal: '',
    selectSituation: '',
    'TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')': '',
  };

  get getName() { // pegar o select Local
    return Metadata.getInstance<Select>('selectName').value || '';
  }

  get getLocal() { // pegar o select Local
    return Metadata.getInstance<Select>('selectLocal').value || '';
  }

  get getSituation() { // pegar o select Local
    return Metadata.getInstance<Select>('selectSituation').value || '';
  }

  get getDate() { // pegar o select Local
    return Metadata.getInstance<Select>('selectDate').value || '';
  }

  public filter() {
    this.dataFilter.selectName = this.getName;
    this.dataFilter.selectLocal = this.getLocal;
    this.dataFilter.selectSituation = this.getSituation;
    this.dataFilter['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] = (this.getDate != null) ? `TO_CHAR('${this.getDate}')` : null;
    this.createCardsRequests();
    this.closeModal();
  }

  // Função que seta os locais nos campos de select "local" presentes em todas as telas de Solicitacao
  public setLocal({ component }:{ component:Select }) {
    Http.get('/LocalRequest?parent=3').then((response: IDictionary) => {
      response.data.dataset.local.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMLOCALCLIENTE,
          value: element.NRLOCALCLIENTE,
        });
      });
    });
  }

  public setSituation({ component }:{ component:Select }) {
    Http.get('/getSituationRequest').then((response: IDictionary) => {
      response.data.dataset.situation.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.IDSITUACAO,
          value: element.NRIDSITUACAO,
        });
      });
    });
  }

  public setNameRequest({ component }:{ component:Select }) {
    Http.get('/getNameRequest').then((response: IDictionary) => {
      response.data.dataset.nameRequest.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMSOLICITACAO,
          value: element.NRSOLICITACAO,
        });
      });
    });
  }

  // Função para criar uma nova solicitação de tarefa
  public async createNewRequest() {
    const form = Metadata.getInstance<Form>('input-form');
    if (form.validate()) {
      LoadingService.show();
      const imagensConvertidas = await FileUpload.converterImagens('FileInputSolicitacao');
      await Http.post('/NewRequest', {
        NMSOLICITACAO: Metadata.getInstance<TextInput>('input-nmsolicitacao').value,
        NRLOCALCLIENTE: Metadata.getInstance<Select>('select-local').value,
        DSSOLICITACAO: Metadata.getInstance<Textarea>('input-text-area').value,
        IDSITUACAO: 1,
        IMAGENS: imagensConvertidas,
        CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      }).then(() => {
        Utilities.showSuccessMessage();
        Router.replace('/solicitacoesTarefas');
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      Utilities.alertMessage('Preencha todos os campos obrigatórios (*)!', 'error');
    }
  }

  // Função que expõe na tela os cards "Minhas Solicitações"
  public async createCardsRequests() {
    LoadingService.show();
    this.hierarquiasController.mountHierarchyUsersArray().then((retorno: IDictionary) => {
      Metadata.getInstance<IterableComponentRender>('icard-tasks-requests').datasource.data = [];
      Http.post('/getCardsRequests', { filter: this.dataFilter, usersIds: retorno }).then((response: IDictionary) => {
        response.data.dataset.cardsRequest.forEach((data: IDictionary) => {
          const dateTime = data.DTSOLICITACAO.split(' ');
          Metadata.getInstance<IterableComponentRender>('icard-tasks-requests').datasource.data.push({
            NRSOLICITACAO: data.NRSOLICITACAO,
            NMSOLICITACAO: data.NMSOLICITACAO,
            NMLOCALCLIENTE: data.NMLOCALCLIENTE,
            DTSOLICITACAO: `${dateTime[0]} - ${dateTime[1]}`,
            IDSITUACAO: data.IDSITUACAO,
          });
        });
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      });
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Função para fazer o botão "Avaliar" ficar invisível nos cards que não estão aguardando avaliação
  public funcButtonVisible({ component }:{ component: Button }) {
    component.isVisible = (component.name.split('_')[2] === 'Aguardando Avaliação');
  }

  // Função para estilizar os status das solicitações
  public stylizeStatus({ component }: IEventParam<Text>) {
    if (component.text === 'Aguardando Avaliação') {
      component.text = "<span  style='color: #ff6700; font-size: 12px;'>AGUARDANDO AVALIAÇÃO</span>";
    }
    if (component.text === 'Concluído') {
      component.text = "<span  style='color: #2b9348; font-size: 12px;'>CONCLUÍDO</span>";
    }
    if (component.text === 'Em andamento') {
      component.text = "<span  style='#dee2e6; color: #023e8a; font-size: 12px;'>EM ANDAMENTO</span>";
    }
    if (component.text === 'Pendente') {
      component.text = "<span  style='color: #ff6700; font-size: 12px;'>PENDENTE</span>";
    }
  }

  // Função que retorna os dados dos cards quando se clica em um, em uma nova tela como exibição
  public showRequest() {
    const NRSOLICITACAO = Number(window.location.search.slice(1));

    Http.post('/showRequest', { NRSOLICITACAO }).then((response: IDictionary) => {
      response.data.dataset.show.forEach((data: IDictionary) => {
        if (data.IDSITUACAO !== 'Aguardando Avaliação') {
          Metadata.getInstance<Button>('show-button-evaluate').disabled = true;
        }
        const dateTime = data.DTSOLICITACAO.split(' ');
        Metadata.getInstance<TextInput>('input-title').value = data.NMSOLICITACAO;
        Metadata.getInstance<TextInput>('input-local').value = data.NMLOCALCLIENTE;
        Metadata.getInstance<TextInput>('input-date').value = `${dateTime[0]} - ${dateTime[1]}`;
        Metadata.getInstance<TextInput>('input-situation').value = data.IDSITUACAO;
        Metadata.getInstance<Textarea>('input-text-area').value = data.DSSOLICITACAO;
      });
    });
    Metadata.getInstance<Button>('show-button-evaluate').to = `/avaliarSolicitacao?${NRSOLICITACAO}`;
  }

  public changeColor({ component }:{ component: Button }) {
    component.outline = false;
    Metadata.getInstance<Button>((component.name === 'button-no') ? 'button-yes' : 'button-no').outline = true;
  }

  public evaluateRequest() {
    const bYes = Metadata.getInstance<Button>('button-yes');
    const bNo = Metadata.getInstance<Button>('button-no');
    const EVALUATION = Metadata.getInstance<ZdRatings>('ratings').value;
    const nr = Number(window.location.search.slice(1));

    if ((!bYes.outline || !bNo.outline) && EVALUATION) {
      Http.post('/evaluateRequest', {
        NRSOLICITACAO: nr,
        IDSATISFACAOSOLICITA: EVALUATION,
        DSSOLICITACAO: Metadata.getInstance<Textarea>('input-text-area').value,
        CONDITION: !bYes.outline,
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
        setTimeout(() => {
          Router.replace('/solicitacoesTarefas');
        }, 2000);
      });
    } else {
      Utilities.alertMessage('Preencha todos os campos obrigatórios (*)!', 'error');
    }
  }
}
